#ifndef __ALLORDERGENERATOR_HH__
#define __ALLORDERGENERATOR_HH__

#include "Generator.hh"

/// Run all order simulation for given t values.

class AllOrderGenerator : public Generator {
public:
  AllOrderGenerator(int seed, Parton::Flavour flav, double eps, double t)
    : Generator(seed,flav), _eps(eps), _t(0.), _tmax(t) {
    _events.push_back(Event(1.));
    _events.back().push_back(Parton(flav));
  }

  AllOrderGenerator(int seed, Parton::Flavour flav, double eps, double t,
		    double ca, double cf, double tr, double nf)
    : Generator(seed,flav,ca,cf,tr,nf), _eps(eps), _t(0.), _tmax(t) {
    _events.push_back(Event(1.));
    _events.back().push_back(Parton(flav));
  }

  ~AllOrderGenerator() {}

  /// set the maximum t value
  void set_tmax(double tmax) {_tmax=tmax;}
  /// set the starting t value
  void set_tmin(double tmin) {_t=tmin;}

  /// set _t=0 and reset partons.
  void reset() {
    _t=0.;
    _events.back().clear();
    _events.back().push_back(Parton(_flav));
  }

protected:
  double _eps,_t,_tmax;
};

#endif // __ALLORDERGENERATOR_HH__
