#ifndef __FIXEDORDERGENERATOR_HH__
#define __FIXEDORDERGENERATOR_HH__

#include "Generator.hh"
#include <iostream>

/// Run fixed order simulation.
class FixedOrderGenerator : public Generator {
public:
  FixedOrderGenerator(int seed, Parton::Flavour flav, int order)
    : Generator(seed,flav),_order(order) {_init();}
  FixedOrderGenerator(int seed, Parton::Flavour flav, int order,
		      double ca, double cf, double tr, double nf)
    : Generator(seed,flav,ca,cf,tr,nf),_order(order) {_init();}

  ~FixedOrderGenerator() {}

  /// reset events
  void reset() {
    _events.clear();
  }

  /// run splitting of a quark or gluon at a given order
  void next_event_group() {
    (this->*split)();
  }
  void set_tmax(double tmax) {}

  void set_tmin(double tmin) {}

private:
  int _order;
  
  /// Regulates divergences near x=1 and x=0 in the
  /// splitting functions
  static const double _eps;

  /// Pointer to a split_[quark|gluon]N function
  void (FixedOrderGenerator::*split)();

  void split_quark1();
  void split_gluon1();
  void split_quark2();
  void split_gluon2();
  void split_quark3();
  void split_gluon3();
  void split_quark4();
  void split_gluon4();
  void split_quark5();
  void split_gluon5();
  
  /// Returns the sign of x
  int sign(double x) {
    return (x > 0) ? 1 : ((x < 0) ? -1 : 0);
  }

  /// q->qg splitting function
  double pqq(const double x) {
    if (x+_eps>1) return 0;
    return _CF*((1+x*x)/(1-x)); 
  }
  /// g->qq splitting function
  double pqg(const double x) {
    return  _nf*_TR*(x*x+(1-x)*(1-x)); 
  }
  /// g->gg splitting function
  double pgg(const double x) {
    if ((x+_eps>1) || (x<_eps)) return 0;
    return _CA*(x/(1-x)+(1-x)/x+x*(1-x)); 
  }

  /// Sets split() to the appropriate function
  void _init() {
    switch(_order) {
    case 1: 
      if (_flav==Parton::q)
	split=&FixedOrderGenerator::split_quark1;
      if (_flav==Parton::g)
	split=&FixedOrderGenerator::split_gluon1;
      break;
    case 2: 
      if (_flav==Parton::q)
	split=&FixedOrderGenerator::split_quark2;
      if (_flav==Parton::g)
	split=&FixedOrderGenerator::split_gluon2;
      break;
    case 3: 
      if (_flav==Parton::q)
	split=&FixedOrderGenerator::split_quark3;
      if (_flav==Parton::g)
	split=&FixedOrderGenerator::split_gluon3;
      break;
    case 4: 
      if (_flav==Parton::q)
	split=&FixedOrderGenerator::split_quark4;
      if (_flav==Parton::g)
	split=&FixedOrderGenerator::split_gluon4;
      break;
    case 5: 
      if (_flav==Parton::q)
	split=&FixedOrderGenerator::split_quark5;
      if (_flav==Parton::g)
	split=&FixedOrderGenerator::split_gluon5;
      break;
    }
  }
};



#endif // __FIXEDORDERGENERATOR_HH__
