#ifndef __OUTPUT_HH__
#define __OUTPUT_HH__

#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include "AverageAndError.hh"
#include "NLOHist.hh"

/// Write output in a file or in std::cout
class Output {
public:
  Output(std::string output, std::string command, std::string time)
    : filename(output), cmdline(command),timestamp(time) {}

  /// return true if i is a writing step
  bool writing_step(int i) {
    if (i<10000000 && i%1000000==0) return true;
    if (i%10000000==0) return true;
    return false;
  }

  /// Write a group of averages and histograms to a file or to std::cout
  void write(std::vector< std::map<std::string,AverageAndError> > averages,
	     std::vector< std::vector<NLOHist> > histos,
	     int nev, double t, double dt, double nsteps, std::string s) {
    // If input.output = "" write to std::cout, else write in file input.output
    std::ofstream ofs;
    std::streambuf * buf;
    if(filename!="") {
      ofs.open(filename.c_str());
      buf = ofs.rdbuf();
    } else {
      buf = cout.rdbuf();
    }
    std::ostream out(buf);
    out.precision(15);
    if(filename!="") 
      out << "# " << cmdline << "\n# " << timestamp << "\n";
    out<<"# Nev = "<<nev<<", "<<s<<" with nsteps = "<<nsteps<<"\n";
    for(int k=0;k<nsteps;++k) { 
      if(nsteps>1 && s!="")
	out << "# "<< s << "= " << t+dt*k << "\n";
      // Write all values to file
      write(out, averages[k]);
      write(out, histos[k], nev);
    }
    if(filename!="")
      ofs.close();    
  }

  /// write averages and histograms to a file or to std::cout
  void write(std::map<std::string,AverageAndError> averages,
	std::vector<NLOHist> histos, int nev) {
    // If input.output = "" write to std::cout, else write in file input.output
    std::ofstream ofs;
    std::streambuf * buf;
    if(filename!="") {
      ofs.open(filename.c_str());
      buf = ofs.rdbuf();
    } else {
      buf = cout.rdbuf();
    }
    std::ostream out(buf);
    out.precision(15);
    if(filename!="") 
      out << "# " << cmdline << "\n# " << timestamp << "\n";
    out << "# Nev = " << nev <<"\n";
    // Write all values to file
    write(out, averages);
    write(out, histos, nev);
    if(filename!="")
      ofs.close();
  }

  /// write averages in ostream
  void write(std::ostream& out, std::map<std::string,AverageAndError> averages) {
    // Write all values to file
    typedef std::map<std::string, AverageAndError>::const_iterator avgitr;
    for (avgitr it = averages.begin(); it != averages.end(); it++) {
      out << "# <" << it->first << "> = " << it->second.average() 
	   << " +- " << it->second.error() << "\n";
    }
    out << "\n";
  }

  /// write histograms in ostream
  void write(std::ostream& out, std::vector<NLOHist> histos, int nev) {
    // Write all values to file
    int n=histos.size();
    for(int i=0;i<n;++i) {
      histos[i].write(out,i);
      out << "\n\n";
    }
  }
  
  /// write full particle configuration in configfile
  std::ostream& write(std::ostream& out, const std::vector<Event>& events) {
    out << "<group>\n";
    for(std::vector<Event>::const_iterator it=events.begin(); 
	it!=events.end(); it++) {
      out << "<event w=" << it->weight() << ">\n";
      int n=it->size();
      for(int i=0; i<n; ++i) {
	if ((*it)[i].flav()==Parton::q) out << "q ";
	if ((*it)[i].flav()==Parton::g) out << "g ";
	out << (*it)[i].x() << "\n";
      }
      out << "</event>\n";
    }
    out << "</group>\n";
    return out;
  }
private:
  std::string filename, cmdline,timestamp;
};

#endif // __OUTPUT_HH__
